@@ -17,13 +17,13 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -810,7 +810,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
       continue;
     if ((IsGlob(filename) == MagickFalse) && (*option != '@'))
       continue;
-    if (*option != '@')
+    if ((*option != '@') && (IsPathAccessible(option) == MagickFalse))
       {
         /*
           Generate file list from wildcard filename (e.g. *.jpg).
@@ -1059,7 +1059,6 @@ MagickExport MagickBooleanType GetExecutionPath(char *path,const size_t extent)
       count;
 
     count=0;
-    execution_path=(char *) NULL;
     program_name=program_invocation_name;
     if (*program_invocation_name != '/')
       {
@@ -1169,7 +1168,7 @@ MagickExport MagickBooleanType GetPathAttributes(const char *path,
       errno=EINVAL;
       return(MagickFalse);
     }
-  (void) ResetMagickMemory(attributes,0,sizeof(struct stat));
+  (void) memset(attributes,0,sizeof(struct stat));
   status=stat_utf8(path,(struct stat *) attributes) == 0 ? MagickTrue :
     MagickFalse;
   return(status);
@@ -1210,13 +1209,16 @@ MagickExport void GetPathComponent(const char *path,PathType type,
   char *component)
 {
   char
-    magick[MaxTextExtent],
-    *q,
-    subimage[MaxTextExtent];
+    *q;
 
   register char
     *p;
 
+  size_t
+    magick_length,
+    subimage_offset,
+    subimage_length;
+
   assert(path != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",path);
   assert(component != (char *) NULL);
@@ -1225,8 +1227,33 @@ MagickExport void GetPathComponent(const char *path,PathType type,
       *component='\0';
       return;
     }
-  (void) CopyMagickString(component,path,MaxTextExtent);
-  *magick='\0';
+  (void) CopyMagickString(component,path,MagickPathExtent);
+  subimage_length=0;
+  subimage_offset=0;
+  if (type != SubcanonicalPath)
+    {
+      p=component+strlen(component)-1;
+      q=strrchr(component,'[');
+      if ((strlen(component) > 2) && (*p == ']') && (q != (char *) NULL) &&
+          ((q == component) || (*(q-1) != ']')) &&
+          (IsPathAccessible(path) == MagickFalse))
+        {
+          /*
+            Look for scene specification (e.g. img0001.pcd[4]).
+          */
+          *p='\0';
+          if ((IsSceneGeometry(q+1,MagickFalse) == MagickFalse) &&
+              (IsGeometry(q+1) == MagickFalse))
+            *p=']';
+          else
+            {
+              subimage_length=(size_t) (p-q);
+              subimage_offset=(size_t) (q-component+1);
+              *q='\0';
+            }
+        }
+    }
+  magick_length=0;
 #if defined(__OS2__)
   if (path[1] != ":")
 #endif
@@ -1241,45 +1268,25 @@ MagickExport void GetPathComponent(const char *path,PathType type,
         if (*p == '\0')
           break;
       }
-    if ((p != component) && (*p == ':') && (IsPathDirectory(path) < 0) &&
-        (IsPathAccessible(path) == MagickFalse))
+    if ((p != component) && (*p == ':') && (IsPathDirectory(component) < 0) &&
+        (IsPathAccessible(component) == MagickFalse))
       {
         /*
           Look for image format specification (e.g. ps3:image).
         */
-        (void) CopyMagickString(magick,component,(size_t) (p-component+1));
-        if (IsMagickConflict(magick) != MagickFalse)
-          *magick='\0';
+        *p='\0';
+        if (IsMagickConflict(component) != MagickFalse)
+          *p=':';
         else
-          for (q=component; *q != '\0'; q++)
-            *q=(*++p);
+          {
+            magick_length=(size_t) (p-component+1);
+            for (q=component; *(++p) != '\0'; q++)
+              *q=(*p);
+            *q='\0';
+          }
         break;
       }
   }
-  *subimage='\0';
-  p=component;
-  if (*p != '\0')
-    p=component+strlen(component)-1;
-  if ((*p == ']') && (strchr(component,'[') != (char *) NULL) &&
-      (IsPathAccessible(path) == MagickFalse))
-    {
-      /*
-        Look for scene specification (e.g. img0001.pcd[4]).
-      */
-      for (q=p-1; q > component; q--)
-        if (*q == '[')
-          break;
-      if (*q == '[')
-        {
-          (void) CopyMagickString(subimage,q+1,MaxTextExtent);
-          subimage[p-q-1]='\0';
-          if ((IsSceneGeometry(subimage,MagickFalse) == MagickFalse) &&
-              (IsGeometry(subimage) == MagickFalse))
-            *subimage='\0';
-          else
-            *q='\0';
-        }
-    }
   p=component;
   if (*p != '\0')
     for (p=component+strlen(component)-1; p > component; p--)
@@ -1289,21 +1296,27 @@ MagickExport void GetPathComponent(const char *path,PathType type,
   {
     case MagickPath:
     {
-      (void) CopyMagickString(component,magick,MaxTextExtent);
+      if (magick_length != 0)
+        (void) CopyMagickString(component,path,magick_length);
+      else
+        *component='\0';
       break;
     }
     case RootPath:
     {
-      for (p=component+(strlen(component)-1); p > component; p--)
-      {
-        if (IsBasenameSeparator(*p) != MagickFalse)
+      if (*component != '\0')
+        {
+          for (p=component+(strlen(component)-1); p > component; p--)
+          {
+            if (IsBasenameSeparator(*p) != MagickFalse)
+              break;
+            if (*p == '.')
+              break;
+          }
+          if (*p == '.')
+            *p='\0';
           break;
-        if (*p == '.')
-          break;
-      }
-      if (*p == '.')
-        *p='\0';
-      break;
+        }
     }
     case HeadPath:
     {
@@ -1313,41 +1326,43 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     case TailPath:
     {
       if (IsBasenameSeparator(*p) != MagickFalse)
-        (void) CopyMagickMemory((unsigned char *) component,
-          (const unsigned char *) (p+1),strlen(p+1)+1);
+        (void) CopyMagickString(component,p+1,MagickPathExtent);
       break;
     }
     case BasePath:
     {
       if (IsBasenameSeparator(*p) != MagickFalse)
-        (void) CopyMagickString(component,p+1,MaxTextExtent);
-      for (p=component+(strlen(component)-1); p > component; p--)
-        if (*p == '.')
-          {
-            *p='\0';
-            break;
-          }
+        (void) CopyMagickString(component,p+1,MagickPathExtent);
+      if (*component != '\0')
+        for (p=component+(strlen(component)-1); p > component; p--)
+          if (*p == '.')
+            {
+              *p='\0';
+              break;
+            }
       break;
     }
     case ExtensionPath:
     {
       if (IsBasenameSeparator(*p) != MagickFalse)
-        (void) CopyMagickString(component,p+1,MaxTextExtent);
-      p=component;
-      if (*p != '\0')
+        (void) CopyMagickString(component,p+1,MagickPathExtent);
+      if (*component != '\0')
         for (p=component+strlen(component)-1; p > component; p--)
           if (*p == '.')
             break;
       *component='\0';
       if (*p == '.')
-        (void) CopyMagickString(component,p+1,MaxTextExtent);
+        (void) CopyMagickString(component,p+1,MagickPathExtent);
       break;
     }
     case SubimagePath:
     {
-      (void) CopyMagickString(component,subimage,MaxTextExtent);
+      *component='\0';
+      if ((subimage_length != 0) && (magick_length < subimage_offset))
+        (void) CopyMagickString(component,path+subimage_offset,subimage_length);
       break;
     }
+    case SubcanonicalPath:
     case CanonicalPath:
     case UndefinedPath:
       break;
